package com.ss.android.ad.report


import android.content.Context
import android.os.Looper
import android.os.Message
import com.bytedance.common.utility.collection.WeakHandler
import com.ss.adnroid.common.ad.AdEventConstant
import com.ss.adnroid.common.ad.MobAdClickCombiner
import com.ss.android.ad.AdDependManager
import com.ss.android.adsupport.model.BaseAdEventModel
import com.ss.android.utils.DeviceUtils

/**
 * DeepLink延迟埋点处理类
 * 需求：https://wiki.bytedance.net/pages/viewpage.action?pageId=173669020
 *
 *
 * Created by chenhaonan on 2018/3/6.
 */

class DeepLinkEventManager private constructor() : WeakHandler.IHandler {

    companion object {
        private val MSG_CHECK_AND_SEND_EVENT = 1
        private val DELAY_TIME = 5000L

        val instance: DeepLinkEventManager = DeepLinkEventManager()
    }

    private val mEventHandler: WeakHandler = WeakHandler(Looper.getMainLooper(), this)

    override fun handleMsg(msg: Message) {
        if (msg.what == MSG_CHECK_AND_SEND_EVENT) {
            val eventModel = msg.obj as BaseAdEventModel
            val isSplash: Boolean = (msg.arg1 == 1)
            // 发deeplink_success埋点有两种情况：
            // 1) 点击调起后第5秒发现App不在前台
            // 2) 点击调起后第5秒发现App在前台，但距离上次在前台的时间不到5秒，说明中间跳出头条又返回了
            if (!AdDependManager.inst().isAppForeground ||
                    ((System.currentTimeMillis() - AdDependManager.inst().lastForegroundStamp() <= DELAY_TIME) && !isSplash)) {
                sendEmbededAdEvent(AdDependManager.inst().context, eventModel, "deeplink_success")
            } else {
                checkIsFail(eventModel)
                // 小米手机拦截窗口出现超过5s，则上报deeplink_success埋点
                if ((DeviceUtils.isMiui() && DeeplinkInterceptHepler.inst().isOnPauseStatus)) {
                    sendEmbededAdEvent(AdDependManager.inst().context, eventModel, "deeplink_success")
                } else {
                    sendEmbededAdEvent(AdDependManager.inst().context, eventModel, "deeplink_failed")
                }
            }
        }
    }

    fun checkAndSendDeepLinkEvent(eventModel: BaseAdEventModel, isSplash: Boolean) {
        val msg = mEventHandler.obtainMessage()
        msg.what = MSG_CHECK_AND_SEND_EVENT
        msg.obj = eventModel
        if (isSplash) {
            msg.arg1 = 1
        }
        mEventHandler.sendMessageDelayed(msg, DELAY_TIME)
    }

    fun checkAndSendDeepLinkEvent(eventModel: BaseAdEventModel) {
        checkAndSendDeepLinkEvent(eventModel, false)
    }

    /**
     * 检测SCHEME跳转是否被拦截，是则打点
     * */
    fun checkIsFail(eventModel: BaseAdEventModel) {

        // 目前只针对小米手机监测，该埋点由于新需求的加入，可能造成数量偏少，准确的数量应该是deeplink_xiaomi_intercept_failed+click_open_app_cancel
        if (DeviceUtils.isMiui() && DeeplinkInterceptHepler.inst().isOnPauseStatus) {
            sendEmbededAdEvent(AdDependManager.inst().context, eventModel, "deeplink_xiaomi_intercept_failed")
        }
    }

    fun sendEmbededAdEvent(context: Context, baseAdEventModel: BaseAdEventModel?, label: String) {
        // 对于广告Post的广告事件
        if (baseAdEventModel != null && baseAdEventModel.isValid) {
            val tag = if (baseAdEventModel.tag.isNullOrEmpty()) "embeded_ad" else baseAdEventModel.tag
            MobAdClickCombiner.onAdEvent(context, tag, label, baseAdEventModel.adId, baseAdEventModel.logExtra, AdEventConstant.AD_EVENT_POSITION_UNKNOWN)
        }
    }

}
