package com.ss.android.auto.opt

import android.content.Context
import android.support.v7.widget.RecyclerView
import android.util.AttributeSet
import android.view.SurfaceView
import android.view.View
import android.view.ViewStub
import android.widget.*
import com.ss.android.auto.opt.apt.CustomViewCreator
import com.ss.android.baseframework.activity.AutoBaseActivity
import com.ss.android.view.SvgCompatAlphaImageView
import com.ss.android.view.SvgCompatCenterTextView
import com.ss.android.view.SvgCompatTextView

class ViewCreateOpt {
    companion object {

        @JvmStatic
        fun init() {
            AutoBaseActivity.sOnViewPreCreateInterceptor = object : AutoBaseActivity.OnViewPreCreateInterceptor {
                override fun onPreCreate(name: String?, context: Context?, attrs: AttributeSet?): View? {
                    return onCreateView(name, context, attrs)
                }
            }
        }

        @JvmStatic
        private fun onCreateView(name: String?, context: Context?, attrs: AttributeSet?): View? {
            if (name == null || context == null || attrs == null) {
                return null
            }
            var view: View? = tryInterceptSystemView(name, context, attrs)

            if (view == null) {
                view = CustomViewCreator.createView(name, context, attrs)
            }

            if (view != null) {
                ViewOpt.log("===> 命中 $name ", 0)
            } else {
                ViewOpt.log("===> 未命中 $name ", 0)
            }

            return view
        }

        private fun tryInterceptSystemView(name: String, context: Context, attrs: AttributeSet): View? {
            /**
             * 这里就不返回AppCompat开头了，原则上不到执行到这里，但是防止一些预加载布局特殊原因执行到这里，防止用的不是AppCompat Theme导致崩溃
             */
            return when (name) {
                "LinearLayout" -> LinearLayout(context, attrs)
                "RelativeLayout" -> RelativeLayout(context, attrs)
                "FrameLayout" -> FrameLayout(context, attrs)
                "ImageButton" -> ImageButton(context, attrs) // should not here
                "ImageView" -> ImageView(context, attrs) // should not here
                "CheckBox" -> CheckBox(context, attrs) // should not here
                "ScrollView" -> ScrollView(context, attrs)
                "TextView" -> TextView(context, attrs) // should not here
                "RadioButton" -> RadioButton(context, attrs) // should not here
                "View" -> View(context, attrs)
                "GridView" -> GridView(context, attrs)
                "ProgressBar" -> ProgressBar(context, attrs)
                "android.support.v7.widget.RecyclerView" -> RecyclerView(context, attrs)
                "com.ss.android.view.SvgCompatTextView" -> SvgCompatTextView(context, attrs)
                "com.ss.android.view.SvgCompatAlphaImageView" -> SvgCompatAlphaImageView(context, attrs)
                "com.ss.android.view.SvgCompatCenterTextView" -> SvgCompatCenterTextView(context, attrs)
                "ViewStub" -> ViewStub(context, attrs) // 这个也有ViewStubCompat，不过运行时没被v7拦截？
                "TabWidget" -> TabWidget(context, attrs)
                "SurfaceView" -> SurfaceView(context, attrs)
                else -> {
                    null
                }
            }
        }

    }
}